# coding: utf-8
"""
    HeadAnomalies.py

    Copyright (c) 2016-2017, Masatsuyo Takahashi, KEK-PF
"""

import numpy        as np
from scipy          import stats

DEBUG = False
ACCEPTABLE_HEAD_RG_RATIO    = 3         # BL_6A/pH6/averaged/pH6_00127_sub.dat

class HeadAnomalies:
    def __init__( self, y, X, Y ):
        self.size   = y.shape[0]
        self.y      = y
        self.y10    = np.percentile( y, 10 )
        self.X      = X
        self.Y      = Y
        self.f0     = 0
        self.t0     = self.size - 1

    def remove_head_anomalies( self ):
        w_ = np.where( self.y < self.y10 )[0]
        e_ = w_[ w_ < self.size/4 ]
        if DEBUG: print( 'size=', self.size, ', e_=', e_ )

        e_size = len( e_ )
        if DEBUG: print( 'e_size=', e_size )
        self.e_size = e_size

        if e_size > 0 and np.max( e_ ) / e_size > 1/2:
            if DEBUG: print( 'need to remove starting error data' )
            start_point = None
            last_error  = None
            max_error   = None
            for i in range( e_size//2, min( e_size * 2, self.size ) ):
                error = np.std( self.y[i:i+e_size] )
                # print( i, error )
                if max_error is None or error > max_error:
                    max_error = error

                if last_error is not None and error < max_error/5:
                    if abs( error / last_error - 1 ) < 0.1:
                        start_point = i - 1
                        break

                last_error = error

            if DEBUG: print( 'start_point=', start_point )
            # assert( start_point is not None )
            if start_point is not None:
                self.f0     = start_point
            else:
                # cases such as sim_data-0_1.1.dat
                pass

        if self.f0 == 0:
            start_point = self.find_start_point()
            # assert( start_point is not None )
            if start_point is not None:
                self.f0 = start_point

        self.start_point = self.f0
        if DEBUG: print( 'self.start_point=', self.start_point )
        self.size   = self.t0 - self.f0 + 1

    def find_start_point( self ):

        start_point, g_neg_len_array = self.find_start_point_impl( 8 )

        if start_point is not None and start_point < 70:
            return start_point

        avg0 = np.average( g_neg_len_array[0:5] )
        avg1 = np.average( g_neg_len_array[-5:] )
        avg_ratio = avg0 / avg1

        if DEBUG: print( 'avg0, avg1=', ( avg0, avg1 ), avg_ratio )

        if avg_ratio > 0.5:
            if avg_ratio > 0.8:
                num_ok = 6
            else:
                num_ok = 7
        else:
            # this is needed for BL-10C/OA_Ald/averaged, etc.
            num_ok = np.max( g_neg_len_array )
            if DEBUG: print( 'try num_ok=', num_ok )

        start_point, _ = self.find_start_point_impl( num_ok )

        if DEBUG: print( 'retried start_point=', start_point )

        return start_point

    def find_start_point_impl( self, num_ok ):
        start_point = None
        g_neg_len_array  = []
        i_limit_init = self.size // 2       # i.e., //8 from the whole size
        i_limit_Rg = None
        for i in range( i_limit_init ):
            g10_        = np.gradient( self.y[i:i+10] )
            g_neg_len   = len( np.where( g10_ < 0  )[0] )
            if DEBUG: print( 'g[%d]=%d' % ( i, g_neg_len ) )
            g_neg_len_array.append( g_neg_len )
            if g_neg_len >= num_ok:
                # calculate the ratio (Rg / i_limit_Rg) to avoid too large Rg at the head region
                Rg = self.evaluate_rg( i )
                if i_limit_Rg is None:
                    for n in [ 4, 3, 2 ]:
                        i_limit = self.size // n    # i.e., //4 //n from the whole size
                        i_limit_Rg = self.evaluate_rg( i_limit )
                        if i_limit_Rg is not None:
                            break
                if DEBUG: print( 'Rg=', Rg, 'i_limit_Rg=', i_limit_Rg )
                if Rg is not None and i_limit_Rg is not None:
                    ratio = Rg / i_limit_Rg
                    if DEBUG: print( 'ratio=', ratio )
                    if ratio  <= ACCEPTABLE_HEAD_RG_RATIO:
                        start_point = i
                        break

        return start_point, g_neg_len_array

    def evaluate_rg( self, i ):
        X, Y = self.X[i:i+10], self.Y[i:i+10]
        slope, intercept, r_value, p_value, std_err = stats.linregress( X, Y )
        rg = np.sqrt( 3*(-slope) ) if slope < 0 else None

        return rg
