# -*- coding: utf-8 -*-
"""

    ファイル名：   Build.py

    処理内容：

        配布形式のビルド

"""

from __future__ import division, print_function, unicode_literals
import sys
import os
import re
import glob
import platform
import shutil
from subprocess         import call
from RequiredPackages   import check_tested
from BasicUtils         import clear_dirs_with_retry, rename_with_retry

class_name_dict = {
    'synthesizer'       :   'Synthesizer',
    'autorg_kek'        :   'AutorgKek',
    'serial_analyzer'   :   'SerialAnalyzer',
    }

def build( thisdir, updir, script ):
    check_tested()

    main_script = script
    main_name   = main_script.replace( '.py', '' )

    bitness = platform.architecture()[0]
    if bitness == '64bit':
        assert( updir.find( 'x64' ) >= 0 )
    elif bitness == '32bit':
        assert( updir.find( 'x86' ) >= 0 )
    else:
        assert( False )

    print( 'Targeted architecture bitness seems to be consistent.' )

    ret = call( 'upx -V > nul', shell=True )
    if ret == 0:
        print( 'UPX is available.' )
        upx_is_avalable = True
    else:
        print( 'UPX is not available.' )
        upx_is_avalable = False

    built_dir = './dist/%s' % ( main_name )
    temp_dirs = [ 'build', 'dist', 'temp' ]
    for dir_ in temp_dirs:
        if os.path.exists( dir_ ):
            shutil.rmtree( dir_ )

    if main_name == 'executables':
        ret = call( 'pyinstaller %s --onedir' % ( main_script ), shell=True )
    else:
        class_name = class_name_dict[main_name]
        if main_name == 'autorg_kek':
            windowed = ''
        else:
            windowed = '--windowed'
        ret = call( 'pyinstaller %s --onedir %s --icon=../lib/%s/%s.ico' % ( main_script, windowed, class_name, main_name ), shell=True )

    print( 'built_dir=', built_dir )

    assert( ret == 0 )
    assert( os.path.exists( built_dir ) )
    """
        mannually execute pyinstaller
        and see if "failed to "failed to create process" is shown.
    """

    python_install_dir = os.path.dirname( sys.executable )

    print( "Python is installed in '%s'" % ( python_install_dir ) )

    import numpy.core
    numpy_core_dir = os.path.dirname( numpy.core.__file__ )
    print( "numpy.core is in '%s'" % ( numpy_core_dir ) )

    if upx_is_avalable:
        working_dll_dir = 'temp'
        clear_dirs_with_retry( [ working_dll_dir ] )
    else:
        working_dll_dir = numpy_core_dir

    existing_dll_names = []
    dll_names = [ 'libiomp5md.dll', 'mkl_avx2.dll', 'mkl_core.dll', 'mkl_def.dll', 'mkl_intel_thread.dll' ]
    for name in dll_names:
        dll_path = os.path.join( numpy_core_dir, name )
        if os.path.exists( dll_path ):
            existing_dll_names.append( name )
            if upx_is_avalable:
                print( 'Compressing %s with UPX...' % ( name ) )
                working_dll_path = os.path.join( working_dll_dir, name )
                shutil.copy2( dll_path, working_dll_path )
                ret = call( 'upx %s' % ( working_dll_path ) )
                assert( ret == 0 )

    for name in existing_dll_names:
        shutil.copy2( os.path.join( working_dll_dir, name ), built_dir )

    if upx_is_avalable:
        import matplotlib
        matplotlib_dir = os.path.dirname( matplotlib.__file__ )

        badly_compressed_dlls = [
            os.path.join( python_install_dir,   'vcruntime140.dll' ),
            os.path.join( matplotlib_dir,       'msvcp140.dll'     ),
            ]

        for path in badly_compressed_dlls:
            name = os.path.split( path )[-1]
            target_path = os.path.join( built_dir, name )
            upxname = name.replace( '.dll', '-upx.dll' )
            rename_path = os.path.join( built_dir, upxname )
            os.rename( target_path, rename_path )
            shutil.copy2( path, target_path )

    dist_dir = os.path.join( updir, main_name )
    if os.path.exists( dist_dir ):
        shutil.rmtree( dist_dir )

    rename_with_retry( built_dir, dist_dir )

    for dir_ in temp_dirs:
        if os.path.exists( dir_ ):
            shutil.rmtree( dir_ )

    os.remove( main_script.replace( '.py', '.spec' ) )

    if upx_is_avalable:
        with_phrase = ' with UPX compress'
    else:
        with_phrase = ''
    print( 'The distribution directory has been successfully built%s.' % ( with_phrase ) )

arc_dict = { '32bit':'x86', '64bit':'x64' }

def split_application( app=None, out_folder=None ):
    if app is not None and app != 'autorg_kek':
        return

    import AutorgKek
    from AppVersion import get_com_version_string
    vstr = get_com_version_string()
    def sub_func(m):
        global arch
        arch = arc_dict[m.group(2)]
        return m.group(1) + '-' + arch

    ver = re.sub( r'(autorg_kek\s+\d+\.\d+\.\d+).+\s+(\d+bit).+', sub_func, vstr ).replace('.', '_').replace(' ', '-')
    print( __file__ )
    assert( __file__.find( arch ) > 0 )
    print( ver, arch )
    path = __file__.replace( '\\', '/' )
    org_folder  = '/'.join( path.split( '/' )[0:-3] )
    up_folder   = '/'.join( path.split( '/' )[0:-4] )

    if out_folder is None:
        out_folder = up_folder + '/' + ver

    print( 'org_folder=', org_folder )
    print( 'out_folder=', out_folder )

    clear_dirs_with_retry( [ out_folder ] )

    for file in [ 'autorg_kek.bat', 'autorg_kek_client.bat' ]:
        print( 'Copying ' + file )
        src_path = org_folder + '/' + file
        tgt_path = out_folder + '/' + file
        shutil.copy2( src_path, tgt_path )

    # todo 'lib/KekLib', 'executables'
    for subfolder in [ 'build', 'glue_devel', 'lib/KekLib', 'lib/AutorgKek', 'test_autorg_kek', 'executables' ]:
        print( 'Copying ' + subfolder )
        src_path = org_folder + '/' + subfolder
        tgt_path = out_folder + '/' + subfolder
        shutil.copytree( src_path, tgt_path )

    # remove/build/*
    for node in [   'build/__pycache__',
                    'build/split.py',
                    'build/serial_analyzer.py',
                    'build/executables.py',
                    'build/synthesizer.py',
                    'executables/synthesizer*',
                    'executables/serial_analyzer*',
                ]:
        tgt_path = out_folder + '/' + node

        if tgt_path.find('*') > 0:
            # print( 'glob ' + tgt_path )
            files = glob.glob(tgt_path)
            for f in files:
                print( 'Removing ' + f )
                # os.remove( f )
            continue

        if not os.path.exists( tgt_path ):
            continue

        print( 'Removing ' + tgt_path )
        if os.path.isdir(tgt_path):
            shutil.rmtree( tgt_path )
        else:
            os.remove( tgt_path )
